/*
 * Drawing routines for xpartinfo
 */

#include "xpartinfo.h"
#include "colors.h"

int IsBoxCorner(node)
     int node;
{
  if (node > NONODES-NCOLS || (node+1)%NCOLS == 0)
    return (0);
  if (NodeUseTable[node] != NodeUseTable[node+1])
    return (0);
  if (NodeUseTable[node] != NodeUseTable[node+NCOLS])
    return (0);
  if (NodeUseTable[node] != NodeUseTable[node+NCOLS+1])
    return (0);
  return (1);
}
void NodeRowCol(node, i, j)
     int node;
     int *i, *j;
{
  *i = nodeColumn[node];
  *j = nodeRow[node];
}

static GC        theGC = NULL;
static XGCValues valsGC;

void GetTheGC()
{
  extern Widget partitionW;

  theGC      = XtGetGC(partitionW, 0, &valsGC);
}

void setColor(color)
     int color;
{
  extern Pixmap partitionPixmap;
  extern Widget partitionW;

  Display *theDisplay;

  theDisplay = XtDisplay(partitionW);

  if (monochrome) {
    XSetStipple(theDisplay, theGC, stipples[color]);
    XSetForeground(theDisplay, theGC, colors[FG_COLOR]);
    XSetFillStyle(theDisplay, theGC, FillOpaqueStippled);
  } else {
    XSetForeground(theDisplay, theGC, colors[color]);
  }
}

void DrawFrame(color, nPPN)
     int color, nPPN;
{
  int i, x, xx, y, yy;
  Display *theDisplay;
  Window theWindow;

  extern Pixmap partitionPixmap;
  extern Widget partitionW;

  theDisplay = XtDisplay(partitionW);
  theWindow  = partitionPixmap; 

  y  = 0;
  yy = nPPN*NROWS;

  setColor(color);

  for (i = 1; i< NCOLS ; i++) {
    x = i*nPPN;
    XDrawLine(theDisplay, theWindow, theGC, x, y, x, yy);
    if( i%4 == 0) {
      x++;
      XDrawLine(theDisplay, theWindow, theGC, x, y, x, yy);
    }
  }

  x = 0;
  xx = nPPN*NCOLS;

  for (i = 1; i< NROWS ; i++) {
    y = i*nPPN;
    XDrawLine(theDisplay, theWindow, theGC, x, y, xx, y);
    if( i%4 == 0) {
      y++;
      XDrawLine(theDisplay, theWindow, theGC, x, y, xx, y);
    }
  }
}

void DrawNode(node, color, nPPN)
     int node;
     int color;
     int nPPN;
{
  int i, j, x, y, ww;
  Display *theDisplay;
  Window theWindow;

  extern Pixmap partitionPixmap;
  extern Widget partitionW;

  theDisplay = XtDisplay(partitionW);
  theWindow  = partitionPixmap; /* XtWindow(partitionW); */

  setColor(color);

  NodeRowCol(node, &i, &j);
  x  = i*nPPN + 2;
  y  = j*nPPN + 2;
  ww = nPPN - 4;
  XFillRectangle(theDisplay, theWindow, theGC, x, y, ww, ww);
}

void DrawPartition(partition, color, nPPN)
     char *partition;
     int color;
     int nPPN;
{
  char *ptr;
  int i, j, corner, node;
  int n1,n2;
  int save1, save2;

  setColor(color);

  ptr = partition;
  if (ptr)
  while (*ptr != '\0') {
    if ((i = sscanf(ptr, "(%d,%d)%d", &n1, &n2, &corner)) == 3) {
      drawPart(nPPN,corner,n1,n2);
      while (*ptr != '\0' && *ptr != ')') ptr++;
      if (*ptr == ')') {
	while (*ptr != '\0' && *ptr != ',') ptr++;
	if (*ptr == ',') ptr++;
      }
    }
    else if ((i = sscanf(ptr, " %dx%d:%d",&n1,&n2, &corner)) == 3) {
      if (n1 && n2) {
	if (n1 == 1) {
	  if (corner < NONODES-NCOLS) {
	    for (j=corner; j<corner+n2; j++)
	      if (NodeUseTable[j] != NodeUseTable[j+NCOLS])
		break;
	    if (j == corner+n2) n1++;
	  }
	  if (corner >= NCOLS) {
	    for (j=corner; j<corner+n2; j++)
	      if (NodeUseTable[j] != NodeUseTable[j-NCOLS])
		break;
	    if (j == corner+n2) {
	      corner -= NCOLS;
	      n1++;
	    }
	  }
	}
	drawPart(nPPN,corner,n1,n2);
      }
      while (*ptr != '\0' && *ptr != ' ') ptr++;
      if (*ptr == ' ') ptr++;
    }
    else if ((i = sscanf(ptr, "%d..%d",&corner,&node)) == 2) {
      while (*ptr != '\0' && *ptr != ' ') ptr++;
      if (*ptr == ' ') ptr++;
      save1 = corner; save2 = node;
      while ((i = sscanf(ptr, "%d..%d",&n1,&n2)) == 2) {
	if ( (n1-corner)%NCOLS || (n2-node)%NCOLS )
	  break;
	while (*ptr != '\0' && *ptr != ' ') ptr++;
	if (*ptr == ' ') ptr++;
	save1 = n1; save2 = n2;
      }
      drawPart(nPPN,corner, 1+(save1-corner)/NCOLS, 1+save2-save1);
    }
    
    else if ((i = sscanf(ptr, "%d",&n1)) == 1) {
      if (IsBoxCorner(n1))
	drawPart(nPPN,n1,2,2);
      else {
	if (n1<NONODES-NCOLS && NodeUseTable[n1] == NodeUseTable[n1+NCOLS])
	  drawPart(nPPN,n1,2,1);
	if (n1>=NCOLS && NodeUseTable[n1] == NodeUseTable[n1-NCOLS])
	  drawPart(nPPN,n1-NCOLS,2,1);
        if (n1%NCOLS && NodeUseTable[n1] == NodeUseTable[n1-1])
	  drawPart(nPPN,n1-1,1,2);
      }
      drawPart(nPPN,n1,1,1);
      while (*ptr != '\0' && *ptr != ' ') ptr++;
      if (*ptr == ' ') ptr++;
    }
    else {
      fprintf(stderr, "Illegal type format %s\n", ptr);
      break;
    }
  }
}

drawPart(nPPN,corner,rows,cols)
      int nPPN;
      int corner;
      int rows;
      int cols;
{
  int i, j, x, xw, y, yw;
  Display *theDisplay;
  Window theWindow;

  extern Pixmap partitionPixmap;
  extern Widget partitionW;

  theDisplay = XtDisplay(partitionW);
  theWindow  = partitionPixmap; /* XtWindow(partitionW); */

  NodeRowCol(corner, &i, &j);
  x  = i*nPPN + 2;
  xw = cols*nPPN - 4;
  y  = j*nPPN + 2;
  yw = rows*nPPN - 4;
  XFillRectangle(theDisplay, theWindow, theGC, x, y, xw, yw);
}

void Erase(color)
     int color;
{
  int nargs;
  Dimension xw, yw;
  Display *theDisplay;
  Window   theWindow;
  Arg args[2];

  extern Pixmap partitionPixmap;
  extern Widget partitionW;

  theDisplay = XtDisplay(partitionW);
  theWindow  = partitionPixmap; /* XtWindow(partitionW); */

  nargs = 0;
  XtSetArg(args[nargs], XtNwidth,  &xw); nargs++;
  XtSetArg(args[nargs], XtNheight, &yw); nargs++;
  XtGetValues(partitionW, args, nargs);

  setColor(color);

  XFillRectangle(theDisplay, theWindow, theGC, 0, 0, xw, yw);
}

void ShowSpecialNodes(color, nPPN)
     int color, nPPN;
{
  int i;

  for (i = 0; i < noSpecialNodes; i++)
    DrawNode(specialNodes[i], color, nPPN);
}

void ShowAllocatedPartitions(user, color3, color2, color, nPPN, nPart)
     char *user;
     int color3, color2, color, nPPN, nPart;
{
  int partition;
  char *theType, *theUser;
  extern struct allocatedPartition *Parts;

  for (partition = 0; partition < nPart; partition++) {
    theType = Parts[partition].Nodes;
    theUser = Parts[partition].Users;
    if (strcmp("0",theUser) == 0)
      DrawPartition(theType, color3, nPPN);
    else if (strcmp(user,theUser) == 0)
      DrawPartition(theType, color2, nPPN);
    else
      DrawPartition(theType, color, nPPN);
  }
}

void ShowPartInfo(nPPN, user, nPart)
     int nPPN;
     char *user;
     int nPart;
{
  int color, color2, color3;

  Erase(BG_COLOR);
  color  = MIN(SPN_COLOR,noColors);
  ShowSpecialNodes(color, nPPN);
  color  = MIN(PTN_COLOR,noColors);
  color2 = MIN(UPT_COLOR,noColors);
  color3 = MIN(EPT_COLOR,noColors);
  ShowAllocatedPartitions(user, color3, color2, color, nPPN, nPart);
  color  = MIN( FG_COLOR,noColors);
  DrawFrame(color, nPPN);
}
