/*
 * xpartinfo: X11 wrapper around partinfo -s
 *
 * Original X11 version by 
 *   Roy Wiliams          (roy@ccsf.caltech.edu)                 Jan 1992
 *
 * ported/expanded to X11/Xt/Xaw by 
 *   Sylvain G. Korzennik (sylvain@ccsf.caltech.edu)             Feb 1992
 *
 * Added StripChart options
 *   Sylvain G. Korzennik (sylvain@ccsf.caltech.edu)             Jun 1992
 *
 * fixed `tloc = time(0);' in TheTimeAndDate() in util.c
 * (Thx to John Salmon)                                          Jul 1992
 *
 * fixed `fclose(file)' missing in GetPartInfo() in getnodes.c
 * (Thx to John Salmon)                                          Jul 1992
 *
 * Upgrade to work with NX/M 1.4
 * Added scrollbar for message window
 * On connection timeout to the Paragon, show last message of 
 * status file if this is updated
 *   Jan Lindheim (lindheim@ccsf.caltech.edu)                    Aug 1992
 *
 * Port to paragon at SDSC J. David Morgenthaler (jdm@sdsc.edu)  Oct 1993
 *
 * Added better fictional partition message, fixed size problem  Dec 1993
 *   J. David Morgenthaler (jdm@sdsc.edu)
 */

char *versionID[] = {"Ver 3.1", "02-Dec-93"};

#include "xpartinfo.h"
#include "colors.h"
#include "xpartinfoP.h"
#include <strings.h>
#include <X11/Shell.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Form.h>

void MakeWidgetTree(), DoUpdate(), Update(), Handle(), Quit(), Resetpartlist();

static Pixmap paragonUpIcon,     paragonDownIcon;
#ifdef SHAPE
static Pixmap paragonUpIconMask, paragonDownIconMask;
#endif
void SetIconParagonUp(), SetIconParagonDown();

#define MAXBUFFER 256
char   message[MAXBUFFER];
struct allocatedPartition *Parts;
int Maxparts = STARTNJOBS;
Pixmap partitionPixmap;
Widget messageW, partitionW;
static Widget topShellW;

static theResourcesStruct      theResources;
static theMonoResourcesStruct  theMonoResources;
static theColorResourcesStruct theColorResources;
static Boolean updating = False;

#ifndef NO_SIGNAL_CATCH
#include <signal.h>
void CatchSig(sig, code, scp, addr)
/* 
 * catch signals handler
 */
     int sig,code;
     struct sigcontext *scp;
     char *addr;
{
  switch(sig) {
  case SIGILL:
    fprintf(stderr,"%s: Illegal instruction (%d, %d at 0x%x)\n",
            progName, sig, code, addr);
    break;
  case SIGFPE:
    fprintf(stderr,"%s: Arithmetic exception (%d, %d at 0x%x)\n",
            progName, sig, code, addr);
    break;
  case SIGBUS:
    fprintf(stderr,"%s: Bus error (%d, %d at 0x%x)\n",
            progName, sig, code, addr);
    break;
  case SIGSEGV:
    fprintf(stderr,"%s: Segmentation violation (%d, %d at 0x%x)\n",
            progName, sig, code, addr);
    break;
  }
    exit (-1);
} /* end of CatchSig */
#endif


void ShowUsage(status)
/*
 * display usage list, and exit with status
 */
     int status;
{
  extern char *versionID[];
  char **ptr;

  fprintf(stderr, "\n\n%s %s/%s\n\n", progName, versionID[0], versionID[1]);
  fprintf(stderr, "Usage: %s [-toolkitoption ...] [-options ...]\n\n", 
          progName);
  for (ptr=usageOptions; *ptr; ptr++) {
    fprintf(stderr,"%s\n",*ptr);
  }
  fprintf(stderr, "\n");
  exit (status);
} /* end of ShowUsage */


void MakeIcons()
{
#include "paragonup.icon"
#include "paragondown.icon"
#ifdef SHAPE
#include "paragonupmask.icon"
#include "paragondownmask.icon"
#endif

  paragonUpIcon = XCreateBitmapFromData(
       XtDisplay(topShellW), XtWindow(topShellW), 
       paragonup_bits, paragonup_width, paragonup_height);
  paragonDownIcon = XCreateBitmapFromData(
       XtDisplay(topShellW), XtWindow(topShellW), 
       paragondown_bits, paragondown_width, paragondown_height);
#ifdef SHAPE
  paragonUpIconMask = XCreateBitmapFromData(
       XtDisplay(topShellW), XtWindow(topShellW), 
       paragonupmask_bits, paragonupmask_width, paragonupmask_height);
  paragonDownIconMask = XCreateBitmapFromData(
       XtDisplay(topShellW), XtWindow(topShellW),
        paragondownmask_bits, paragondownmask_width, paragondownmask_height);
#endif
}


main(argc, argv)
     int   argc;
     char *argv[];
{
  XEvent event;
  int screen;

	if ( gethostid() == 0x51007664 )
	{
		fprintf( stderr, "Please run xpartinfo on any SGI or Sun except Systems1\n" );
		exit( 1 );
	}

  /* save the program name as a global variable */
  progName = strcpy((char *) malloc(strlen(argv[0])+1), argv[0]);

#ifndef NO_SIGNAL_CATCH
  /* catch following signals: */
  signal(SIGILL,  CatchSig);
  signal(SIGFPE,  CatchSig);
  signal(SIGBUS,  CatchSig);
  signal(SIGSEGV, CatchSig);
#endif

  /* initialize the top level Xt shell and parse the options */
  topShellW = XtInitialize(progName, className,
                           options, XtNumber(options),
                           &argc, argv);

  /* do we have invalid args? */
  if (argc > 1) {
    int i;
    fprintf(stderr,"%s: invalid, ambiguous or incomplete arg(s): ",progName);
    for (i = 1; i < argc; i++) fprintf (stderr, "%s ", argv[i]);
    ShowUsage(1);
  }

  /* retrieve the resources from the RDB */
  XtGetApplicationResources(topShellW, &theResources,
                            resources, XtNumber(resources),
                            NULL, 0);

  /* was the "-help" option given? */
  if (theResources.showUsage) ShowUsage(0);

  /* cannot use nPPN < 6 */
  if(theResources.nPPN < 6) theResources.nPPN = 6;

  /* check if monochrome display */
  screen = XDefaultScreen(XtDisplay(topShellW));
  monochrome = (XDisplayPlanes(XtDisplay(topShellW), screen) == 1 ||
                theResources.monochrome );

  /* retrieve the mono/color resources from the RDB */
  if (monochrome)
    XtGetApplicationResources(topShellW, &theMonoResources,
                              monoResources, XtNumber(monoResources),
                              NULL, 0);
  else
    XtGetApplicationResources(topShellW, &theColorResources,
                              colorResources, XtNumber(colorResources),
                              NULL, 0);

  /* build the widget tree */
  (void) MakeWidgetTree(topShellW,theResources.nPPN);

  /* realize the widget tree */
  (void) XtRealizeWidget(topShellW);

  /* icon pixmaps */
  (void) MakeIcons();
  (void) SetIconParagonUp();

  if (monochrome) {
    for (noColors=0; noColors<NO_COLORS; noColors++)
      stipples[noColors] = 
        XCreateBitmapFromData(XtDisplay(topShellW), XtWindow(topShellW),
                              &hatch[8*(noColors)], 8, 8);
    noColors = NO_COLORS;
  } else {
    /* get the color levels */
    colors[ BG_COLOR] = theColorResources.bg;
    colors[ FG_COLOR] = theColorResources.fg;
    colors[SPN_COLOR] = theColorResources.spn;
    colors[PTN_COLOR] = theColorResources.ptn;
    colors[MXC_COLOR] = theColorResources.mxc;
    colors[UPT_COLOR] = theColorResources.upt;
    colors[EPT_COLOR] = theColorResources.ept;
    noColors = NO_COLORS;
  }

  /* retrieve the "special nodes" map */
  GetSpecialNodeMap(theResources.specialNodeMapFileName);

  /* build the node nums map */
  (void) MakeNodeMap();

  /* initialize the partition table */
  Parts = (struct allocatedPartition *) 
           calloc(Maxparts*sizeof(struct allocatedPartition));

  /* show the partition info in 300usec */
  XtAddTimeOut(300, DoUpdate, &updating);

  /* Set up a loop to wait for X event */
  while (True) {
    XtNextEvent(&event);
    XtDispatchEvent(&event);
  }

}  /* end of main */


#include <X11/Xaw/Box.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/StripChart.h>

#define XCMW     XtCreateManagedWidget
#define NARGMAX  8
Arg args[NARGMAX];

void GetLoadValue(widget, clientData, loadValue) 
     Widget widget;
     XtPointer clientData;
     double *loadValue;
{
  static double storedLoadValue = 0;

  storedLoadValue = actualLoadValue/20.0;
/* 20*0.05 = 1 secs --> most likely invalid --> full scale */
  if (storedLoadValue <  0.05 ||
      storedLoadValue > LOAD_SC_SCALE) storedLoadValue = LOAD_SC_SCALE;

  *loadValue = storedLoadValue;
}

void GetNoNodesValue(widget, clientData, noNodesValue) 
     Widget widget;
     XtPointer clientData;
     double *noNodesValue;
{
  static double storedNoNodesValue;

  storedNoNodesValue = ((double)NONODES_SC_SCALE*totalNoNodesUsed)/NONODES;
  *noNodesValue = storedNoNodesValue;
}

static XtCallbackRec getLoadValueCallback[] = {
  {GetLoadValue, NULL}, {NULL, NULL} };

static XtCallbackRec getNoNodesValueCallback[] = {
  {GetNoNodesValue, NULL}, {NULL, NULL} };

void MakeWidgetTree(parentW)
/*
 * build the widget tree:
 *
 * topBox              Box (vertical)
 *   buttonBox           Box (horizontal)
 *     quitButton          Command
 *     updateButton        Command
 *   message             AsciiText
 *   partBox             Box (horizontal)
 *     space               fill up left of partition display
 *     partition           Label (display a pixmap that contains `partinfo')
 *   load                StripChart
 *   nonodes             StripChart
 *
 */
     Widget parentW;
{
  int nargs;
  int width;
  Widget   topBoxW, buttonBoxW, quitButtonW, updateButtonW, 
           loadStripChartW, noNodesStripChartW, spaceW, partBoxW;
  Display *theDisplay;
  Window   theRootWindow;

  width = (NCOLS>MINWIDTH?NCOLS:MINWIDTH)*theResources.nPPN;
  nargs = 0;
  XtSetArg(args[nargs], XtNorientation, XtorientVertical); nargs++;
  topBoxW = XCMW("topBox", boxWidgetClass, parentW, args, nargs);

  nargs = 0;
  XtSetArg(args[nargs], XtNorientation, XtorientHorizontal);  nargs++;
  buttonBoxW  = XCMW("buttonBox",  boxWidgetClass,   topBoxW, args, nargs);

  nargs = 0;
  XtSetArg(args[nargs], XtNwidth,  width/2+2); nargs++;
  updateButtonW = XCMW("update", commandWidgetClass, 
                       buttonBoxW, args, nargs);
  XtAddEventHandler(updateButtonW, ButtonPressMask, FALSE, Update, NULL);

  nargs = 0;
  XtSetArg(args[nargs], XtNwidth,  width/2+2); nargs++;
  quitButtonW = XCMW("quit", commandWidgetClass, 
                     buttonBoxW, args, nargs);
  XtAddEventHandler(quitButtonW, ButtonPressMask, FALSE, Quit, NULL);

  nargs = 0;
  sprintf(message,"Please wait...");
  XtSetArg(args[nargs], XtNstring,  message); nargs++;
  XtSetArg(args[nargs], XtNscrollHorizontal, XawtextScrollAlways); nargs++;
  XtSetArg(args[nargs], XtNwidth,  width+20);  nargs++;
  XtSetArg(args[nargs], XtNheight,  35); nargs++;
  messageW = XCMW("message", asciiTextWidgetClass, topBoxW, args, nargs);

  theDisplay    = XtDisplay(parentW);
  theRootWindow = XDefaultRootWindow(theDisplay);
  partitionPixmap =  XCreatePixmap(theDisplay, theRootWindow,
                                   NCOLS*theResources.nPPN, 
                                   NROWS*theResources.nPPN, 
                                   XDisplayPlanes(theDisplay, 
                                                  XDefaultScreen(theDisplay)));

  if(theResources.showNoNodesSC) {
    updateSC = theResources.updateDelay/2; 
    nargs = 0;
    XtSetArg(args[nargs], XtNheight,    NONODES_SC_HEIGHT); nargs++;
    XtSetArg(args[nargs], XtNwidth,     width+20); nargs++;
    XtSetArg(args[nargs], XtNminScale,  NONODES_SC_SCALE); nargs++;
    XtSetArg(args[nargs], XtNjumpScroll, 1); nargs++;
    XtSetArg(args[nargs], XtNupdate,    updateSC); nargs++;
    XtSetArg(args[nargs], XtNgetValue,  getNoNodesValueCallback); nargs++;
    noNodesStripChartW =  XCMW("nonodes", stripChartWidgetClass, topBoxW,
                               args, nargs);
  }

  nargs = 0;
  XtSetArg(args[nargs], XtNorientation, XtorientHorizontal);  nargs++;
  partBoxW  = XCMW("partBox",  boxWidgetClass,   topBoxW, args, nargs);

 if (NCOLS < MINWIDTH) {
    nargs = 0;
    XtSetArg(args[nargs], XtNinternalHeight,  0); nargs++;
    XtSetArg(args[nargs], XtNinternalWidth,   0); nargs++;
    XtSetArg(args[nargs], XtNheight, NROWS*theResources.nPPN); nargs++;
    XtSetArg(args[nargs], XtNwidth, (MINWIDTH-NCOLS)*theResources.nPPN/2-1); 
    nargs++;
    spaceW  = XCMW("spaceBox",  boxWidgetClass,  partBoxW, args, nargs);
  }

  nargs = 0;
  XtSetArg(args[nargs], XtNlabel,  ""); nargs++;
  XtSetArg(args[nargs], XtNinternalHeight,  0); nargs++;
  XtSetArg(args[nargs], XtNinternalWidth,   0); nargs++;
  XtSetArg(args[nargs], XtNheight, NROWS*theResources.nPPN); nargs++;
  XtSetArg(args[nargs], XtNwidth,  NCOLS*theResources.nPPN); nargs++;
  partitionW  = XCMW("partition",  labelWidgetClass, partBoxW, args, nargs);
  XtAddEventHandler(partitionW,
                    ButtonPressMask | ButtonReleaseMask,
                    False, Handle, &theResources.nPPN);

 if (NCOLS < MINWIDTH) {
    nargs = 0;
    XtSetArg(args[nargs], XtNinternalHeight,  0); nargs++;
    XtSetArg(args[nargs], XtNinternalWidth,   0); nargs++;
    XtSetArg(args[nargs], XtNheight, NROWS*theResources.nPPN); nargs++;
    XtSetArg(args[nargs], XtNwidth, (MINWIDTH-NCOLS)*theResources.nPPN/2-1); 
    nargs++;
    spaceW  = XCMW("spaceBox",  boxWidgetClass,  partBoxW, args, nargs);
  }

  if(theResources.showLoadSC) {
    updateSC = theResources.updateDelay/2; 
    nargs = 0;
    XtSetArg(args[nargs], XtNheight,    LOAD_SC_HEIGHT); nargs++;
    XtSetArg(args[nargs], XtNwidth,     width+20); nargs++;
    XtSetArg(args[nargs], XtNminScale,  LOAD_SC_SCALE); nargs++;
    XtSetArg(args[nargs], XtNjumpScroll, 1); nargs++;
    XtSetArg(args[nargs], XtNupdate,    updateSC); nargs++;
    XtSetArg(args[nargs], XtNgetValue,  getLoadValueCallback); nargs++;
    loadStripChartW =  XCMW("load", stripChartWidgetClass, topBoxW,
                            args, nargs);
  }
}

void SetIconParagonUp()
{
  int nargs = 0;

  XtSetArg(args[nargs], XtNtitle, className); nargs++;
  XtSetArg(args[nargs], XtNiconPixmap, paragonUpIcon); nargs++;
#ifdef SHAPE
  XtSetArg(args[nargs], XtNiconMask,   paragonUpIconMask); nargs++;
#endif
  XtSetValues(topShellW, args, nargs);
}

void SetIconParagonDown()
{
  int nargs = 0;

  XtSetArg(args[nargs], XtNtitle, className); nargs++;
#ifdef SHAPE
  XtSetArg(args[nargs], XtNiconPixmap, paragonDownIcon); nargs++;
#endif
  XtSetArg(args[nargs], XtNiconMask,   paragonDownIconMask); nargs++;
  XtSetValues(topShellW, args, nargs);
}

void RefreshPartition()
{
  int nargs = 0;

  XtSetArg(args[nargs], XtNbitmap, partitionPixmap); nargs++;
  XtSetValues(partitionW, args, nargs);
}

void ShowMessage(message)
     char *message;
{
  int nargs = 0;
  XtSetArg(args[nargs], XtNstring,  message); nargs++;
  XtSetArg(args[nargs], XtNresize, XawtextResizeWidth); nargs++;
  XtSetValues(messageW, args, nargs);
}

void DoUpdate(updating, id)
     Boolean *updating;
     XtIntervalId *id;
{
  if (! *updating) {
    (void) GetTheGC();
    *updating = True;
  }
  (void) Update(NULL, NULL, NULL);
  XtAddTimeOut(theResources.updateDelay*1000, DoUpdate, &updating);
}

void Update(widget, client_data, event)
     Widget widget;
     caddr_t client_data;
     XEvent *event;
{
  int status, nJobs, jobsWaiting;
  int date1, date2;
  int i;
  static prevStatus = -99;
  char *partInfoDate;
  char *statusFileName;
  char *statusFileDate;
  char buffer[MAXBUFFER];
  FILE *file;
  char *DateOfFile();

  sprintf(message,"Updating, please wait...");
  ShowMessage(message); 

  /* load value */
  if (theResources.showLoadSC)
    (void) GetActualLoadValue(theResources.loadValueFileName);

  /* retrieve the part info */
  if ((status = GetPartInfo(theResources.partInfoFileName, &partInfoDate, 
			    &nJobs, &jobsWaiting)) < 0) {

    if (status != prevStatus) (void) SetIconParagonDown();
    Erase(SPN_COLOR);
    if (status == -10) {
      prevStatus = status;
      status = GetPartInfo(theResources.partInfoFileName, 
			    &partInfoDate, &nJobs, &jobsWaiting);
    }
    switch (status){
    case -10:
      sprintf(message,"%.16s Could not open the partinfo file (%s)", 
              partInfoDate, theResources.partInfoFileName);
      break;
    case -1:
      if((file = fopen(theResources.statusFileName, "r")) == NULL) {
        sprintf(message,"%.16s Could not open status file (%s)",
        partInfoDate, theResources.statusFileName);
      }
      else {
        while (fgets(buffer,MAXBUFFER,file) != NULL)
          strcpy(message,buffer);
        fclose(file);
      }
      if ((i=sscanf(buffer,"%d",&date1)) == 1) {
        sscanf(partInfoDate+8,"%d",&date2);
        if (date1 != date2 || strncmp(buffer+3,partInfoDate+4,3) != 0)
          sprintf(message,"%.16s Paragon not responding....", partInfoDate);
      }
      else
        sprintf(message,"%.16s Paragon not responding....", partInfoDate);
      break;
    default:
      sprintf(message,"%.16s Invalid partinfo file (%s) %d", 
              partInfoDate, theResources.partInfoFileName, status);
      break;
    }
    DrawFrame(FG_COLOR, theResources.nPPN);
    totalNoNodesUsed = 0;

  } 
  if (status >= 0) {

  /* now show it */
    ShowPartInfo(theResources.nPPN, theResources.user, status);    
    switch (status) {
    case 0:
      sprintf(message,"No allocated partitions (%.16s).", partInfoDate);
      break;
    default:
      if (jobsWaiting)
	sprintf(message,"%d %s running, %d waiting (%.16s).", nJobs, 
		(nJobs==1?"job":"jobs"), jobsWaiting, partInfoDate);
      else
	sprintf(message,"%d %s running (%.16s).", nJobs, 
		(nJobs==1?"job":"jobs"), partInfoDate); 
      break;
    }
    if (status != prevStatus) (void) SetIconParagonUp();
  }

  RefreshPartition();
  ShowMessage(message); 
  prevStatus = status;
  Resetpartlist();
}
